﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Praktikum_2_Loesung
{
    abstract class Signal
    {
        public virtual double Startzeit { get { return double.PositiveInfinity; } }
        public virtual double Endzeit { get { return double.NegativeInfinity; } }
        public abstract double GibWert(double zeit);
    }

    class Sinus : Signal
    {
        double frequenz;
        double amplitude;

        public Sinus(double frequenz, double amplitude)
        {
            this.frequenz = frequenz;
            this.amplitude = amplitude;
        }

        public override double GibWert(double zeit)
        {
            return amplitude * Math.Sin(2.0 * Math.PI * frequenz * zeit);
        }
    }

    class Sägezahn : Signal
    {
        double frequenz;
        double amplitude;

        public Sägezahn(double frequenz, double amplitude)
        {
            this.frequenz = frequenz;
            this.amplitude = amplitude;
        }

        public override double GibWert(double zeit)
        {
            double u = frequenz * zeit;
            return amplitude * 2.0 * (u - Math.Floor(u) - 0.5);
        }
    }

    class Mischung : Signal
    {
        Signal signal1;
        Signal signal2;
        double anteil1;

        public Mischung(Signal signal1, Signal signal2, double anteil1)
        {
            this.signal1 = signal1;
            this.signal2 = signal2;
            this.anteil1 = anteil1;
        }

        public override double GibWert(double zeit)
        {
            return anteil1 * signal1.GibWert(zeit) + (1.0 - anteil1) * signal2.GibWert(zeit);
        }

        public override double Startzeit { get { return Math.Min(signal1.Startzeit, signal2.Startzeit) ; } }

        public override double Endzeit { get { return Math.Max(signal1.Endzeit, signal2.Endzeit); } }
    }

    class Ableitung : Signal
    {
        Signal signal;
        double deltaT;

        public Ableitung(Signal signal, double deltaT)
        {
            this.signal = signal;
            this.deltaT = deltaT;
        }

        public override double GibWert(double zeit)
        {
            return (signal.GibWert(zeit + 0.5 * deltaT) - signal.GibWert(zeit - 0.5 * deltaT)) / deltaT;
        }

        public override double Startzeit { get { return signal.Startzeit; } }

        public override double Endzeit { get { return signal.Endzeit; } }
    }

    class GespeichertesSignal : Signal
    {
        protected double[] werte;
        double startzeit;
        double endzeit;

        public GespeichertesSignal(double[] werte, double startzeit, double endzeit)
        {
            this.werte = werte;
            this.startzeit = startzeit;
            this.endzeit = endzeit;
        }

        public override double GibWert(double zeit)
        {
            double index = Math.Round((zeit - startzeit) / (endzeit - startzeit) * ( werte.Length - 1.0));
            if(index < 0.0 || index > werte.Length - 1.0)
            {
                return 0.0;
            }
            return werte[(int)index];
        }

        public override double Startzeit { get { return startzeit; } }

        public override double Endzeit { get { return endzeit; } }
    }

    class WeißesRauschen : GespeichertesSignal
    {
        static Random würfel = new Random();

        public WeißesRauschen(double startzeit, double endzeit, double zeitschritt)
            : base(null, startzeit, endzeit)
        {
            int länge = (int)Math.Round((endzeit - startzeit) / zeitschritt + 1.0);
            werte = new double[länge];
            for (int i = 0; i < länge; i++)
            {
                werte[i] = würfel.NextDouble() - 0.5;
            }
        }
    }

    class Integral : GespeichertesSignal
    {
        public Integral(Signal signal, double startzeit, double endzeit, double zeitschritt)
            : base(null, Math.Max(signal.Startzeit, startzeit), Math.Min(signal.Endzeit, endzeit))
        {
            int länge = (int)Math.Round((Endzeit - Startzeit) / zeitschritt + 1.0);
            werte = new double[länge];
            double aktuellerWert = 0.0;
            for (int i = 0; i < länge; i++)
            {
                aktuellerWert += signal.GibWert(Startzeit + i * zeitschritt) * zeitschritt;
                werte[i] = aktuellerWert;
            }
        }
    }
}
